#include "BmpHelper.h"
#include <stdio.h>
#include <iostream>

#ifdef _MSC_VER
#pragma warning( push )
#pragma warning( disable : 4996 ) // 'fopen': This function or variable may be unsafe
#endif
void BmpHelper::SaveToBmpFile(const char* fname, LONG width, LONG height, const unsigned char *buffer)
{
    FILE *f = fopen(fname, "wb");
    if (!f) {
        std::cerr << "Could not write file " << fname << std::endl;
        return;
    }
    WriteBmpHeader(f, width, height);

    const unsigned char zeros[4] = {0, 0, 0};
    LONG lineDataSize = width * GetBytesPerPixel();
    LONG padding = GetBmpRowSize(width) - lineDataSize;
    if (!padding) {
        fwrite(buffer, lineDataSize, height, f);
    } else {
        for (int h = 0; h < height; ++h) {
            fwrite(buffer, lineDataSize, 1, f);
            fwrite(zeros, padding, 1, f);
            buffer += lineDataSize;
        }
    }
    fclose(f);
}
#ifdef _MSC_VER
#pragma warning( pop )
#endif

void BmpHelper::WriteBmpHeader(FILE *f, LONG width, LONG height)
{
    fHeader_.bfSize = (DWORD)(sHeader_ + GetBmpRowSize(width) * height);
    iHeader_.biWidth = (LONG)width;
    iHeader_.biHeight = -(LONG)height;
    fwrite(&fHeader_, sizeof(BITMAPFILEHEADER), 1, f);
    fwrite(&iHeader_, sizeof(BITMAPINFOHEADER), 1, f);
    WriteBmpHeaderExt(f, width, height);
}

void BmpHelper::InitHeaders()
{
    fHeader_.bfType = 'B' + 256*'M'; 
    sHeader_ = (DWORD)(sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + GetBmpHeaderExtSize());
    fHeader_.bfReserved1 = 0;
    fHeader_.bfReserved2 = 0;
    fHeader_.bfOffBits = (DWORD)(sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + GetBmpHeaderExtSize());

    iHeader_.biSize = sizeof(BITMAPINFOHEADER);
    iHeader_.biPlanes = 1;
    iHeader_.biBitCount = 8 * GetBytesPerPixel();
    iHeader_.biCompression = BI_RGB;
    iHeader_.biSizeImage = 0;
    iHeader_.biXPelsPerMeter = 0;
    iHeader_.biYPelsPerMeter = 0;
    iHeader_.biClrUsed = 0;
    iHeader_.biClrImportant = 0;

    InitHeadersExt();
}

LONG BmpHelper::GetBmpRowSize(LONG width) const
{
    // row size must be a multiple of 4 bytes
    LONG rowSize = width * GetBytesPerPixel();
    if (rowSize & 3) {
        rowSize |= 3;
        ++rowSize;
    }
    return rowSize;
}

BmpHelper8::BmpHelper8()
: BmpHelper()
{
    InitHeaders();
}

size_t BmpHelper8::GetBmpHeaderExtSize() const
{
    return 256 * sizeof(RGBQUAD);
}

void BmpHelper8::InitHeadersExt()
{
    for (size_t index = 0; index < 256; ++index)
    {
        p8bpp[index].rgbBlue = (BYTE)index;
        p8bpp[index].rgbGreen = (BYTE)index;
        p8bpp[index].rgbRed = (BYTE)index;
        p8bpp[index].rgbReserved = 0;
    }
}

/** prepends BMP headers in front of width x height array of 0-255 pixel values. */
void BmpHelper8::WriteBmpHeaderExt(FILE *f, LONG width, LONG height)
{
    fwrite(p8bpp, sizeof(RGBQUAD), 256, f);
}

BmpHelper24::BmpHelper24()
: BmpHelper()
{
    InitHeaders();
}
