/* Copyright Euresys 2015 */

#ifndef COAXLINKGRABBER_HEADER_FILE
#define COAXLINKGRABBER_HEADER_FILE

#include <EGrabber.h>

class CoaxlinkGrabber: public Euresys::EGrabber<Euresys::CallbackOnDemand> {
    public:
        CoaxlinkGrabber(Euresys::GenTL &gentl, int interfaceIndex = 0, int deviceIndex = 0, int dataStreamIndex = 0)
        : Euresys::EGrabber<Euresys::CallbackOnDemand>(gentl, interfaceIndex, deviceIndex, dataStreamIndex)
        {}

        // Compatibility
        class Buffer: public Euresys::Buffer {
            public:
                Buffer(const Euresys::NewBufferData &data)
                : Euresys::Buffer(data)
                {}
                // Compatibility
                Buffer(const Euresys::Buffer &data)
                : Euresys::Buffer(data)
                {}
                void Push(Euresys::Internal::EGrabberBase &grabber) {
                    return push(grabber);
                }
                void *GetUserPointer() {
                    return getUserPointer();
                }
        };
        class ScopedBuffer: public Euresys::ScopedBuffer {
            public:
                ScopedBuffer(Euresys::EGrabber<Euresys::CallbackOnDemand> &grabber, uint64_t timeout = GENTL_INFINITE)
                : Euresys::ScopedBuffer(grabber, timeout)
                {}
                ScopedBuffer(Euresys::Internal::EGrabberBase &grabber, const Euresys::NewBufferData &data)
                : Euresys::ScopedBuffer(grabber, data)
                {}
                // Compatibility
                void *GetUserPointer() {
                    return getUserPointer();
                }
        };

        size_t ReallocBuffers(size_t buffer_count, size_t buffer_size = 0) {
            return reallocBuffers(buffer_count, buffer_size);
        }
        void AnnounceBuffer(void *ptr, size_t size, void *userPointer = 0) {
            announceBuffer(ptr, size, userPointer);
        }
        void FlushBuffers(Euresys::gc::ACQ_QUEUE_TYPE operation = Euresys::gc::ACQ_QUEUE_ALL_TO_INPUT) {
            flushBuffers(operation);
        }
        void Start(uint64_t frame_count = GENTL_INFINITE) {
            start(frame_count);
        }
        void Stop() {
            stop();
        }
        size_t GetWidth() {
            return getWidth();
        }
        size_t GetHeight() {
            return getHeight();
        }
        size_t GetPayloadSize() {
            return getPayloadSize();
        }
        std::string GetPixelFormat() {
            return getPixelFormat();
        }
        Buffer Pop(uint64_t timeout = GENTL_INFINITE) {
            return pop(timeout);
        }
        void CancelPop() {
            cancelPop();
        }

};

typedef Euresys::InterfaceModule InterfacePort;
typedef Euresys::DeviceModule DevicePort;
typedef Euresys::StreamModule StreamPort;
typedef Euresys::RemoteModule RemotePort;

#endif
